﻿// ---------------------------------------------------------------------
// This source file is (C) 2010-2013 Sanders Technology http://www.sanderstechnology.com
// Please review the copyright terms in the Conditions.txt file accompanying this source code
// Author:  Rob Sanders
// Email:   rob.sanders@gmail.com
// Created: Thursday, 10 February 2011, Last Modified: 23 October, 2012
// ---------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Reflection;

namespace Common
{
    public static class EventHandlers
    {
        #region ErrorOutputEvents

        /// <summary>
        /// A Generic Way to Log Exceptions
        /// </summary>
        /// <param name="ex"></param>
        public static void HandleExceptionEvent(Exception ex)
        {
            HandleExceptionEvent(ex, String.Empty);
        }

         /// <summary>
        /// A Generic Way to Log Exceptions with a custom message
        /// </summary>
        /// <param name="ex"></param>
        /// <param name="message"></param>
        public static void HandleExceptionEvent(Exception ex, string message)
        {
            HandleExceptionEvent(ex, message, false);
        }

        /// <summary>
        /// A Generic Way to Log Exceptions with a custom message
        /// </summary>
        /// <param name="ex"></param>
        /// <param name="message"></param>
        /// <param name="logStackTrace"></param>
        public static void HandleExceptionEvent(Exception ex, string message, bool logStackTrace)
        {
            if (ExceptionEvent == null)
                return;

            if(String.IsNullOrEmpty(message))
            {
                message = String.Empty;
            }

            string callingFunction = String.Empty;

            StackTrace stackTrace = new StackTrace();
            StackFrame stackFrame = stackTrace.GetFrame(2);
            if (stackFrame != null)
            {
                MethodBase methodBase = stackFrame.GetMethod();
                if (methodBase != null)
                {
                    callingFunction = methodBase.Name;
                }
            }
            
            if (ex.InnerException != null)
            {
                message += String.Format(" {0}::{1}-{2}", callingFunction, ex.Message, ex.InnerException.Message);
                
                if (logStackTrace)
                {
                    message += ex.StackTrace.ToString();
                }

                ExceptionEvent(message, ex);
            }
            else
            {
                message += String.Format(" {0}::{1}", callingFunction, ex.Message);
                
                if (logStackTrace)
                {
                    message += ex.StackTrace.ToString();
                }

                ExceptionEvent(message, ex);
            }
        }

        public static void HandleAsynchEvent()
        {
            if (AsynchProcessingEvent == null)
                return;
            
            AsynchProcessingEvent();            
        }

        public static void HandleFeedbackEvent(string message)
        {
            if (AsynchFeedbackProcessingEvent == null)
                return;
            
            AsynchFeedbackProcessingEvent(message);            
        }

        public delegate void EventHandler(string message, Exception ex);
        public delegate void AsynchFeedbackEvent(string message);
        public delegate void AsynchEvent();
        
        public static event EventHandler ExceptionEvent;
        public static event AsynchFeedbackEvent AsynchFeedbackProcessingEvent;
        public static event AsynchEvent AsynchProcessingEvent;

        #endregion
    }
}
