// ---------------------------------------------------------------------
// This source file is (C) 2010-2013 Sanders Technology http://www.sanderstechnology.com
// Please review the copyright terms in the Conditions.txt file accompanying this source code
// Author:  Rob Sanders
// Email:   rob.sanders@gmail.com
// Created: Thursday, 10 February 2011, Last Modified: 23 October, 2012
// ---------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Globalization;
using System.Threading;
using System.Reflection;
using Common.Logging;

namespace Common
{
    /// <summary>
    /// Used to Trace and Log Error Events
    /// </summary>
    public static class Log
    {
        #region FormatDetails Variant Functions        
       
        public static void Write(LogProviderType type, LogPriority priority, string message)
        {
            switch (type)
            {
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.EventLog:
                {
                    EventLogProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.DebugLog:
                {
                    Debug.Write(message);
                    FlatFileProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.FlatFile:
                {
                    FlatFileProvider.GetInstance().Write(message);
                    break;
                }
                default:
                case LogProviderType.Trace:
                {
                    TraceProvider.Write(message);
                    break;
                }                
            }
        }

        public static void Write(LogProviderType type, string message, Exception ex)
        {
            switch (type)
            {
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.EventLog:
                {
                    EventLogProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.FlatFile:
                {
                    FlatFileProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.DebugLog:
                {
                    string debugMessage = message + " " + ex.Message;
                    if (ex.InnerException != null)
                    {
                        debugMessage += " [" + ex.InnerException.Message + "]";
                    }
                    Debug.Write(debugMessage);
                    FlatFileProvider.GetInstance().Write(debugMessage);
                    break;
                }
                default:
                case LogProviderType.Trace:
                {
                    TraceProvider.Write(message);
                    break;
                }
            }
        }

        public static void Write(LogProviderType type, string message, EventLogEntryType logType)
        {
           
            switch (type)
            {
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().Write(message);
                    break;
                }
                case LogProviderType.EventLog:
                {
                    EventLogProvider.GetInstance().WriteLog(message, logType);
                    break;
                }
                case LogProviderType.DebugLog:
                case LogProviderType.FlatFile:
                {
                    FlatFileProvider.GetInstance().Write(message);
                    break;
                }
                default:
                case LogProviderType.Trace:
                {
                    TraceProvider.Write(message);
                    break;
                }
            }
        }

        public static void WriteLine(LogProviderType type, LogPriority priority, string message)
        {
            WriteLine(type, priority, message, true);
        }

        public static void WriteLine(LogProviderType type, LogPriority priority, string message, bool includeTimestamp)
        {
            switch (type)
            {
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().WriteLine(message);
                    break;
                }
                case LogProviderType.EventLog:
                {
                    EventLogProvider.GetInstance().WriteLine(message);
                    break;
                }
                case LogProviderType.FlatFile:
                {
                    FlatFileProvider.GetInstance().WriteLine(message, includeTimestamp);
                    break;
                }
                default:
                case LogProviderType.Trace:
                {
                    TraceProvider.WriteLine(message);
                    break;
                }
                case LogProviderType.DebugLog:
                {
                    Debug.WriteLine(message);
                    FlatFileProvider.GetInstance().WriteLine(message, includeTimestamp);
                    break;
                }
            }
        }

        /// <summary>
        /// Writes an Exception to the Specified Target
        /// </summary>
        /// <param name="type"></param>
        /// <param name="message"></param>
        /// <param name="ex"></param>
        public static void WriteLine(LogProviderType type, string message, Exception ex)
        {
            string logMessage = String.Empty;

            if (!String.IsNullOrEmpty(message))
            {
                logMessage = message;
            }
            if (ex != null)
            {
                logMessage = String.Format("{0} [Exception: {1}]", logMessage, ex.Message);
            
                if (ex.InnerException != null)
                {
                    logMessage = String.Format("{0} [InnerException: {1}]", logMessage, ex.InnerException.Message);
                }
            }

            switch (type)
            {
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().WriteLine(logMessage);
                    break;
                }
                case LogProviderType.EventLog:
                {
                    EventLogProvider.GetInstance().WriteLine(logMessage);
                    break;
                }
                case LogProviderType.FlatFile:
                {
                    FlatFileProvider.GetInstance().WriteLine(logMessage);
                    break;
                }
                default:
                case LogProviderType.Trace:
                {
                    TraceProvider.WriteLine(logMessage);
                    break;
                }
                case LogProviderType.DebugLog:
                {                   
                    Debug.WriteLine(logMessage);
                    FlatFileProvider.GetInstance().WriteLine(logMessage);
                    break;
                }
            }
        }

        /// <summary>
        /// Writes an entry into the specified target
        /// </summary>
        /// <param name="type"></param>
        /// <param name="message"></param>
        /// <param name="logType"></param>
        public static void WriteLine(LogProviderType type, string message, EventLogEntryType logType)
        {

            switch (type)
            {
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().WriteLine(message);
                    break;
                }
                case LogProviderType.EventLog:
                {
                    EventLogProvider.GetInstance().WriteLog(message, logType);
                    break;
                }
                case LogProviderType.DebugLog:
                case LogProviderType.FlatFile:
                {
                    FlatFileProvider.GetInstance().Write(message);
                    break;
                }
                default:
                case LogProviderType.Trace:
                {
                    TraceProvider.Write(message);
                    break;
                }
            }
        }

        /// <summary>
        /// Uses Diagnostics
        /// </summary>
        /// <param name="message"></param>
        public static void WriteTrace(LogProviderType type, LogPriority priority, string message)
        {
            switch (type)
            {
                default:
                case LogProviderType.Trace:
                {
                    Log.WriteLine(LogProviderType.Trace, LogPriority.Informational, message);
                    break;
                }
                case LogProviderType.FlatFile:
                {
                    if (priority != LogPriority.Low)
                    {
                        FlatFileProvider.GetInstance().WriteLine(message);
                    }
                    break;
                }
                case LogProviderType.DebugLog:
                {
                    Debug.WriteLine(message);
                    if (priority != LogPriority.Low)
                    {
                        FlatFileProvider.GetInstance().WriteLine(message);
                    }
                    break;
                }
                case LogProviderType.Database:
                {
                    //DatabaseProvider.GetInstance().Write(message);
                    break;
                }               
            }            
        }

        /// <summary>
        /// Uses Diagnostics
        /// </summary>
        /// <param name="message"></param>
        public static void WriteDebugTrace(LogProviderType type, string message)
        {
            string callingFunction = String.Empty;

            StackTrace stackTrace = new StackTrace();
            StackFrame stackFrame = stackTrace.GetFrame(2);
            if (stackFrame != null)
            {
                MethodBase methodBase = stackFrame.GetMethod();
                if (methodBase != null)
                {
                    callingFunction = methodBase.Name;
                }
            }

            string threadName = String.Empty;
            if (!String.IsNullOrEmpty(Thread.CurrentThread.Name))
            {
                threadName = Thread.CurrentThread.Name;
            }
            else
            {
                threadName = Thread.CurrentThread.ManagedThreadId.ToString();
            }

            if(!String.IsNullOrEmpty(callingFunction))
            {
                //Log.WriteLine(LogProviderType.Trace, LogPriority.Informational, String.Format("{3}: '{0}': Called From [{1}] on Thread [{2}]", message, callingFunction, threadName, DateTime.Now));
                Log.WriteLine(LogProviderType.Trace, LogPriority.Informational, String.Format("{2}: {0}: Called From [{1}]", message, callingFunction, DateTime.Now));
            }
            else
            {
                //Log.WriteLine(LogProviderType.Trace, LogPriority.Informational, String.Format("{2}: '{0}': executed on Thread [{1}]", message, threadName, DateTime.Now));
                //Log.WriteLine(LogProviderType.Trace, LogPriority.Informational, String.Format("{1}: {0}", message, DateTime.Now));
                TraceProvider.Write(String.Format("{1}: {0}", message, DateTime.Now));
            }
        }

        /// <summary>
        /// This WriteLog is for Tracing Only
        /// </summary>
        /// <param name="message"></param>
        /// <param name="value"></param>
        public static void WriteTrace(LogProviderType type, LogPriority priority, string message, object value)
        {
            if (ConfigManager.GetBooleanValue("TraceLevel", false))
            {
                LogDetails details = new LogDetails();

                details.Message = message;
                
                try
                {
                    details.ErrorMessages = string.Format(CultureInfo.CurrentCulture, "{0}", value);
                }
                catch (Exception ex)
                {
                    details.ErrorMessages = String.Format(CultureInfo.CurrentCulture, "Unable to log the trace value, {0}", ex.Message);
                }

                details.EventLogType = EventLogEntryType.Information;
                details.Severity = 3;
                details.Timestamp = DateTime.Now;

                switch (type)
                {
                    case LogProviderType.Database:
                    {
                        //DatabaseProvider.GetInstance().Write(details);
                        break;
                    }
                    case LogProviderType.EventLog:
                    {
                        EventLogProvider.GetInstance().Write(details);
                        break;
                    }
                    case LogProviderType.DebugLog:
                    {
                        Debug.WriteLine(message, value);
                        FlatFileProvider.GetInstance().Write(details);
                        break;
                    }
                    case LogProviderType.FlatFile:
                    {
                        FlatFileProvider.GetInstance().Write(details);
                        break;
                    }
                    default:
                    case LogProviderType.Trace:
                    {
                        TraceProvider.Write(details);
                        break;
                    }
                }                
            }
        }

        /// <summary>
        /// Applies to the Flat File Provider
        /// </summary>
        /// <param name="name"></param>
        public static void StartNewLogFile(string name)
        {
            FlatFileProvider.GetInstance().ChangeLogFile(name);
        }

        /// <summary>
        /// The location of the Flat File Log
        /// </summary>
        public static string LogFileLocation
        {
            get
            {
                return FlatFileProvider.GetInstance().LogFileLocation;
            }
        }

        /// <summary>
        /// The file name of the Flat File Log
        /// </summary>
        public static string LogFileName
        {
            get
            {
                return FlatFileProvider.GetInstance().LogFileName;
            }
        }

        #endregion        
    }
}