﻿// ---------------------------------------------------------------------
// This source file is (C) 2010-2013 Sanders Technology http://www.sanderstechnology.com
// Please review the copyright terms in the Conditions.txt file accompanying this source code
// Author:  Rob Sanders
// Email:   rob.sanders@gmail.com
// Created: Thursday, 10 February 2011, Last Modified: 23 October, 2012
// ---------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Globalization;
using System.Diagnostics.Eventing.Reader;

namespace Common.Logging
{
    public class EventLogProvider : BaseLogProvider, ILogProvider
    {
        private static EventLogProvider _instance;

        public EventLogProvider()
        {
        }

        public static EventLogProvider GetInstance()
        {
            if (_instance == null)
            {
                _instance = new EventLogProvider();
            }
            return _instance;
        }

        public void InitializeLog()
        {
        }

        public bool Write(string message)
        {
            try
            {
                EventLog.WriteEntry("System", message, EventLogEntryType.Information);
            }
            catch { }
            return true;
        }

        public bool WriteLine(string message)
        {
            try
            {
                EventLog.WriteEntry("System", message, EventLogEntryType.Information);
            }
            catch { } 
            return true;
        }


        public bool Write(LogDetails details)
        {
            WriteLog(details);
            return true;
        }

        /// <summary>
        /// Format a Event Log entry based on an exception
        /// </summary>
        /// <param name="ex"></param>
        public void WriteLog(string message, Exception ex, EventLogEntryType entryType)
        {
            if (ex == null)
                return;

            LogDetails details = new LogDetails();

            if (ex.InnerException != null)
            {
                details.Message = ex.InnerException.ToString();
            }
            else
            {
                details.Message = message;
            }

            details.Severity = 1;
            details.Timestamp = DateTime.Now;
            details.ErrorMessages = ex.StackTrace;
            details.EventLogType = entryType;
            details.Title = String.Format(CultureInfo.CurrentCulture, "{0}: {1}", message, ex.Message);
            details.AppDomainName = ex.Source;

            try
            {
                EventLog.WriteEntry(ConfigManager.GetStringValue("ServiceName", "Windows Service"), FormatMessage(details), details.EventLogType, details.EventId);
            }
            catch { }
        }

        /// <summary>
        /// Format a Event Log entry based on an exception
        /// </summary>
        /// <param name="ex"></param>
        public void WriteLog(string message, Exception ex)
        {
            if (ex == null)
                return;

            LogDetails details = new LogDetails();

            if (ex.InnerException != null)
            {
                details.Message = ex.InnerException.ToString();
            }
            else
            {
                details.Message = message;
            }

            details.Severity = 1;
            details.Timestamp = DateTime.Now;
            details.ErrorMessages = ex.StackTrace;
            details.EventLogType = EventLogEntryType.Error;
            details.Title = String.Format(CultureInfo.CurrentCulture, "{0}: {1}", message, ex.Message);
            details.AppDomainName = ex.Source;

            try
            {
                EventLog.WriteEntry(ConfigManager.GetStringValue("ServiceName", "Windows Service"), FormatMessage(details), details.EventLogType, details.EventId);
            }
            catch { }
        }

        /// <summary>
        /// Format a Event Log entry based on a message fromt the ConfigManager
        /// </summary>
        /// <param name="ex"></param>
        public void WriteConfigLog(string source, string message)
        {   
            LogDetails details = new LogDetails();

            details.Message = message;
            details.Severity = 1;
            details.Timestamp = DateTime.Now;
            details.ErrorMessages = message;
            details.EventLogType = EventLogEntryType.Error;
            details.Title = String.Format(CultureInfo.CurrentCulture, "{0} Configuration Error", source);

            try
            {
                EventLog.WriteEntry(source, FormatMessage(details), details.EventLogType, details.EventId);
            }
            catch { }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public void WriteLog(string message, EventLogEntryType entryType)
        {
            LogDetails details = new LogDetails();

            details.Message = message;
            details.EventLogType = entryType;
            details.Severity = 2;
            details.Timestamp = DateTime.Now;

            WriteLog(details);
        }

        /// <summary>
        /// Writes an EventLog Entry
        /// </summary>
        /// <param name="details"></param>
        public void WriteLog(LogDetails details)
        {
            EventLog.WriteEntry(ConfigManager.GetStringValue("ServiceName", "Windows Service"), FormatMessage(details), details.EventLogType, details.EventId);
        }

        public void Cleanup()
        {
        }       
    }
}
