﻿// ---------------------------------------------------------------------
// This source file is (C) 2010-2013 Sanders Technology http://www.sanderstechnology.com
// Please review the copyright terms in the Conditions.txt file accompanying this source code
// Author:  Rob Sanders
// Email:   rob.sanders@gmail.com
// Created: Thursday, 10 February 2011, Last Modified: 23 October, 2012
// ---------------------------------------------------------------------
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading;
using ThreadManager;
using System.Diagnostics;

namespace ThreadManager.UnitTests
{
    [TestClass]
    public class SynchronousExecutionTests
    {
        private ManualResetEvent _flag;
        private object _lock;
        private int _threadCount = 0;
        private int _totalThreadCount = 0;

        
        protected void package_PackageProcessingEvent(ThreadWorkPackage context)
        {
            //do some work
            Trace.WriteLine(String.Format("{0}:{1}", context.PackageSeries, context.PackageId));
        }

        [TestMethod]
        public void TestAsynchExecution()
        {
            _flag = new ManualResetEvent(false);
            _lock = new object();

            _totalThreadCount = 3;

            ThreadPoolManager mgmt1 = new ThreadPoolManager();
            ThreadPoolManager mgmt2 = new ThreadPoolManager();
            ThreadPoolManager mgmt3 = new ThreadPoolManager();

            mgmt1.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(mgmt_FinishProcessingEvent);
            mgmt2.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(mgmt_FinishProcessingEvent);
            mgmt3.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(mgmt_FinishProcessingEvent);

            Trace.WriteLine("** Task 1 has 2000 Items **");
            Trace.WriteLine("** Task 2 has 1000 Items **");
            Trace.WriteLine("** Task 3 has 1500 Items **");

            mgmt1.BeginExecution(TestCommon.GetWorkList("Task 1", 20, 0, new ThreadWorkPackage.ExecutionEvent(package_PackageProcessingEvent)), false); //do NS lookup
            mgmt3.BeginExecution(TestCommon.GetWorkList("Task 2", 10, 0, new ThreadWorkPackage.ExecutionEvent(package_PackageProcessingEvent)), false); //do WHOIS
            mgmt2.BeginExecution(TestCommon.GetWorkList("Task 3", 15, 0, new ThreadWorkPackage.ExecutionEvent(package_PackageProcessingEvent)), false); //do alerts

            while (!_flag.WaitOne(1000))
            {
                Thread.Sleep(100);
            }
        }

        /// <summary>
        /// Callback from ASync processing
        /// </summary>
        protected void mgmt_FinishProcessingEvent()
        {
            try
            {
                Monitor.Enter(_lock);
                ++_threadCount;
                //Trace.WriteLine(String.Format("Execution Count {0}", _threadCount));

                if (_threadCount == _totalThreadCount)
                {
                    _flag.Set();
                }
            }
            finally
            {
                Monitor.Exit(_lock);
            }
        }
    }
}
