﻿// ---------------------------------------------------------------------
// This source file is (C) 2010-2013 Sanders Technology http://www.sanderstechnology.com
// Please review the copyright terms in the Conditions.txt file accompanying this source code
// Author:  Rob Sanders
// Email:   rob.sanders@gmail.com
// Created: Thursday, 10 February 2011, Last Modified: 23 October, 2012
// ---------------------------------------------------------------------
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading;
using System.Diagnostics;

namespace ThreadManager.UnitTests
{
    /// <summary>
    /// Summary description for TaskQueuingTests
    /// </summary>
    [TestClass]
    public class TaskQueuingTests
    {
        private ThreadPoolManager _instance;
        private bool _finished;

        public TaskQueuingTests()
        {
            _instance = new ThreadPoolManager();
            _instance.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(FinishProcessingEvent);
        }        

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void QueueAbortsProperly()
        {
            AutoResetEvent waitOne = new AutoResetEvent(false);
            AutoResetEvent waitTwo = new AutoResetEvent(false);

            List<ThreadWorkPackage> stuffToDo = new List<ThreadWorkPackage>();
            for (int x = 0; x < 20; ++x)
            {
                ThreadWorkPackage pkg = new ThreadWorkPackage();

                pkg.UseWaitEvent = true;
                pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(TaskQueueingPackageProcessingEvent);
                pkg.Context = this;
                pkg.PackageSeries = "Multiple Item (master abort 1)";

                stuffToDo.Add(pkg);
            }

            _finished = false;

            _instance.BeginExecution(stuffToDo, false, true, waitOne, "Master Abort 1", -1);

            stuffToDo = new List<ThreadWorkPackage>();
            for (int x = 0; x < 20; ++x)
            {
                ThreadWorkPackage pkg = new ThreadWorkPackage();

                pkg.UseWaitEvent = true;
                pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(TaskQueueingPackageProcessingEvent);
                pkg.Context = this;
                pkg.PackageSeries = "Multiple Item (master abort 2)";

                stuffToDo.Add(pkg);
            }

            _instance.BeginExecution(stuffToDo, false, true, waitTwo, "Master Abort 2", -1);

            waitOne.Set();

            while (!_finished)
            {
                Thread.Sleep(100);
            }

            Trace.WriteLine(String.Format("{0}/{1} of {2}", _instance.ExecutedThreads, _instance.AbortedThreads, _instance.TotalThreads));

            Assert.IsTrue(_instance.TotalThreads == 40);
            Assert.IsTrue(_instance.AbortedThreads < 21);
        }

        [TestMethod]
        public void QueueAbortsOneMasterWait()
        {
            AutoResetEvent waitOne = new AutoResetEvent(false);
            AutoResetEvent waitTwo = new AutoResetEvent(false);

            List<ThreadWorkPackage> stuffToDo = new List<ThreadWorkPackage>();
            for (int x = 0; x < 20; ++x)
            {
                ThreadWorkPackage pkg = new ThreadWorkPackage();

                pkg.UseWaitEvent = true;
                pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(TaskQueueingPackageProcessingEvent);
                pkg.Context = this;
                pkg.PackageSeries = "Multiple Item (master abort 1)";

                stuffToDo.Add(pkg);
            }

            _finished = false;

            _instance.BeginExecution(stuffToDo, false, true, waitOne, "Master Abort 1", -1);

            stuffToDo = new List<ThreadWorkPackage>();
            for (int x = 0; x < 10; ++x)
            {
                ThreadWorkPackage pkg = new ThreadWorkPackage();

                pkg.UseWaitEvent = true;
                pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(TaskQueueingPackageProcessingEvent);
                pkg.Context = this;
                pkg.PackageSeries = "Multiple Item (master abort 2)";

                stuffToDo.Add(pkg);
            }

            _instance.BeginExecution(stuffToDo, false, true, waitTwo, "Master Abort 2", - 1);

            waitTwo.Set();
            waitOne.Set();

            while (!_finished)
            {
                Thread.Sleep(100);
            }

            Trace.WriteLine(String.Format("{0}/{1} of {2}", _instance.ExecutedThreads, _instance.AbortedThreads, _instance.TotalThreads));
            
            Assert.IsTrue(_instance.TotalThreads == 30);
            //Assert.IsTrue(_instance.ExecutedThreads > 10);
        }

        protected void FinishProcessingEvent()
        {
            _finished = true;
        }

        protected void TaskQueueingPackageProcessingEvent(ThreadWorkPackage package)
        {   
            Random rdm = new Random(DateTime.Now.Millisecond);
            int seed = rdm.Next(0, 6000);
            Thread.Sleep(seed);
        }
    }
}
