﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Common;
using System.Threading;

namespace ThreadManager.UnitTests
{
    public partial class ThreadForm : Form
    {
        AutoResetEvent _masterReset = new AutoResetEvent(false);
        ThreadPoolManager _mgr;
        List<AutoResetEvent> _events;

        public ThreadForm()
        {
            InitializeComponent();
            EventHandlers.ExceptionEvent += new EventHandlers.EventHandler(EventHandlers_ExceptionEvent);
            _mgr = new ThreadPoolManager();
            _mgr.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(FinishProcessingEvent);
            _events = new List<AutoResetEvent>();
        }

        protected override void OnClosed(EventArgs e)
        {
            EventHandlers.ExceptionEvent -= new EventHandlers.EventHandler(EventHandlers_ExceptionEvent);
            base.OnClosed(e);
        }

        protected void FinishProcessingEvent()
        {
            try
            {
                SendMessage("\r\nFinished! " + _mgr.ExecutedThreads + " of " + _mgr.TotalThreads + " (" + _mgr.AbortedThreads + " aborted)");
                //btnStop.Enabled = false;
                //btnStartThreads.Enabled = true;
            }
            catch (Exception)
            {
            }
        }

        protected void SetText(object sender, EventArgs e)        
        {
            outputText.Text += sender as string;
        }

        protected void EventHandlers_ExceptionEvent(string message, Exception ex)
        {
            SendMessage("Exception Thrown: " + message);
        }

        private void btnCloseForm_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnStartThreads_Click(object sender, EventArgs e)
        {
            try
            {
                //outputText.Text = String.Empty;
                //btnStartThreads.Enabled = false;
                //btnStop.Enabled = true;

                int count = 1;
                Int32.TryParse(threadCount.Text, out count);

                if (count == 1)
                {
                    ThreadWorkPackage pkg = new ThreadWorkPackage();

                    pkg.WaitEvent = _masterReset;
                    pkg.UseWaitEvent = true;
                    pkg.Context = this;

                    pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(ThreadFormPackageProcessingEvent);
                    pkg.PackageAbortEvent += new ThreadWorkPackage.ExecutionAbortEvent(PackageAbortEvent);
                    pkg.PackageSeries = "Single Item (no abort)";

                    _mgr.BeginExecution(pkg, false, true);
                }
                else
                {
                    List<ThreadWorkPackage> stuffToDo = new List<ThreadWorkPackage>();

                    for (int x = 0; x < count; ++x)
                    {
                        ThreadWorkPackage pkg = new ThreadWorkPackage();

                        pkg.UseWaitEvent = true;
                        pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(ThreadFormPackageProcessingEvent);
                        pkg.PackageAbortEvent += new ThreadWorkPackage.ExecutionAbortEvent(PackageAbortEvent);
                        pkg.Context = this;
                        pkg.PackageSeries = "Multiple Item (master abort)";

                        stuffToDo.Add(pkg);
                    }
                    _mgr.BeginExecution(stuffToDo, false, true, _masterReset, "Master Abort",  -1);
                }
            }
            catch (Exception ex)
            {
                EventHandlers.HandleExceptionEvent(ex);
            }
        }

        protected void SendMessage(ThreadWorkPackage package, string message)
        {
            if (package.Context == null)
                return;
            
            ThreadForm thisForm = package.Context as ThreadForm;
            if (thisForm == null)
                return;

            // marshal it to the UI Thread
            object[] pList = { message, System.EventArgs.Empty };
            thisForm.BeginInvoke(new System.EventHandler(SetText), pList);
        }

        protected void SendMessage(string message)
        { 
            // marshal it to the UI Thread
            object[] pList = { message, System.EventArgs.Empty };
            this.BeginInvoke(new System.EventHandler(SetText), pList);
        }

        protected void PackageAbortEvent(ThreadWorkPackage package, bool timeOut)
        {
            //SendMessage(package, "\r\nAbort - Cleanup ");
        }

        protected void ThreadFormPackageProcessingEvent(ThreadWorkPackage package)
        {
            try
            {
                //SendMessage(package, "\r\nExec - " + package.PackageSeries);
                
                Random rdm = new Random(DateTime.Now.Millisecond);
                int seed = rdm.Next(600, 12000);
                Thread.Sleep(seed);

                SendMessage(package, "\r\nComp - " + package.PackageSeries);
                
                _events.Remove(package.WaitEvent as AutoResetEvent);
            }
            catch (ThreadInterruptedException)
            {
                SendMessage(package, "\r\nTerm - " + package.PackageSeries);
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            //btnStartThreads.Enabled = true;
            //btnStop.Enabled = false;
            _masterReset.Set();
        }

        private void btnStartOneKill_Click(object sender, EventArgs e)
        {
            try
            {
                _events.Clear();

                int count = 1;
                Int32.TryParse(threadCount.Text, out count);

                List<ThreadWorkPackage> stuffToDo = new List<ThreadWorkPackage>();

                for (int x = 0; x < count; ++x)
                {
                    ThreadWorkPackage pkg = new ThreadWorkPackage();
                    AutoResetEvent eventWait = new AutoResetEvent(false);

                    pkg.UseWaitEvent = true;
                    pkg.WaitEvent = eventWait;

                    pkg.PackageProcessingEvent += new ThreadWorkPackage.ExecutionEvent(ThreadFormPackageProcessingEvent);
                    pkg.PackageAbortEvent += new ThreadWorkPackage.ExecutionAbortEvent(PackageAbortEvent);
                    pkg.Context = this;
                    pkg.PackageSeries = "Multiple Item (individual abort)";

                    _events.Add(eventWait);
                    stuffToDo.Add(pkg);
                }
                _mgr.BeginExecution(stuffToDo, false, true);

            }
            catch (Exception ex)
            {
                EventHandlers.HandleExceptionEvent(ex);
            }
        }

        private void btnAbort_Click(object sender, EventArgs e)
        {
            if (_events.Count() > 0)
            {
                int count = _events.Count();
                Int32.TryParse(abortItems.Text, out count);

                int abortCount = 0;

                foreach (AutoResetEvent events in _events)
                {
                    if (abortCount >= count)
                        break;

                    events.Set();
                    ++abortCount;
                }
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            outputText.Text = String.Empty;
        }
    }
}
