﻿// ---------------------------------------------------------------------
// This source file is (C) 2010-2013 Sanders Technology http://www.sanderstechnology.com
// Please review the copyright terms in the Conditions.txt file accompanying this source code
// Author:  Rob Sanders
// Email:   rob.sanders@gmail.com
// Created: Thursday, 10 February 2011, Last Modified: 23 October, 2012
// ---------------------------------------------------------------------
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading;
using ThreadManager;
using System.Diagnostics;

namespace ThreadManager.UnitTests
{
    public class TestPackage
    {
        public ThreadPoolManager Manager { get; set; }
        public List<ThreadWorkPackage> Package { get; set; }
    }    

    [TestClass]
    public class ThreadSafetyTests
    {
        private ManualResetEvent _flag;
        private object _lock;
        private int _threadCount = 0;
        private int _totalThreadCount = 0;

        [TestMethod]
        public void UseThreadTestForm()
        {
            ThreadForm form = new ThreadForm();
            form.ShowDialog();
        }

        [TestMethod]
        public void ThreadTest1Synch()
        {            
            Debug.WriteLine(TestThreadSafeAsynchExecution(true));
        }

        [TestMethod]
        public void ThreadTest2ASynch()
        {            
            Debug.WriteLine(TestThreadSafeAsynchExecution(false));
        }

        private TimeSpan TestThreadSafeAsynchExecution(bool synch)
        {
            _flag = new ManualResetEvent(false);
            _lock = new object();

            _totalThreadCount = 3;

            TestPackage package1 = new TestPackage();
            package1.Manager = new ThreadPoolManager();
            package1.Package = TestCommon.GetWorkList("Task Alpha", 15, 0, new ThreadWorkPackage.ExecutionEvent(package_PackageProcessingEvent));

            TestPackage package2 = new TestPackage();
            package2.Manager = new ThreadPoolManager();
            package2.Package = TestCommon.GetWorkList("Task Beta", 5, 10, new ThreadWorkPackage.ExecutionEvent(package_PackageProcessingEvent));

            TestPackage package3 = new TestPackage();
            package3.Manager = new ThreadPoolManager();
            package3.Package = TestCommon.GetWorkList("Task Zeta", 10, 100, new ThreadWorkPackage.ExecutionEvent(package_PackageProcessingEvent));

            package1.Manager.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(mgmt_FinishProcessingEvent);
            package2.Manager.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(mgmt_FinishProcessingEvent);
            package3.Manager.FinishProcessingEvent += new ThreadPoolManager.FinishedProcessing(mgmt_FinishProcessingEvent);

            int y = package1.Package.Count() +
                    package2.Package.Count() +
                    package3.Package.Count();
            
            Stopwatch watch = new Stopwatch();
            watch.Start();

            if (synch)
            {
                for (int x = 0; x < y; ++x)
                {
                    package_PackageProcessingEvent(null);
                }
            }
            else
            {
                ThreadPool.QueueUserWorkItem(ThreadProc, package1);
                ThreadPool.QueueUserWorkItem(ThreadProc, package2);
                ThreadPool.QueueUserWorkItem(ThreadProc, package3);

                while (!_flag.WaitOne(1000))
                {
                    Thread.Sleep(100);
                }
            }           

            watch.Stop();

            if (synch)
            {
                Trace.WriteLine(String.Format("Synchronous Execution Completed in {0} for {1} items", watch.Elapsed, y));
            }
            else
            {
                Trace.WriteLine(String.Format("Asynchronous Execution Completed in {0} for {1} items", watch.Elapsed, y));
            }

            return watch.Elapsed;
        }

        private static void ThreadProc(object stateInfo)
        {
            TestPackage package = stateInfo as TestPackage;
            package.Manager.BeginExecution(package.Package, false);
        }

        protected void package_PackageProcessingEvent(ThreadWorkPackage context)
        {
            //do some work
            Random rdm = new Random(DateTime.Now.Millisecond);
            int seed = rdm.Next(100, 3000);
            Thread.Sleep(seed); //simulate a bit of work
            //Trace.WriteLine(String.Format("{0}:{1}", context.PackageSeries, context.PackageId));
        }

        /// <summary>
        /// Callback from ASync processing
        /// </summary>
        protected void mgmt_FinishProcessingEvent()
        {
            try
            {
                Monitor.Enter(_lock);
                ++_threadCount;
                //Trace.WriteLine(String.Format("Execution Count {0}", _threadCount));

                if (_threadCount == _totalThreadCount)
                {
                    _flag.Set();
                }
            }
            finally
            {
                Monitor.Exit(_lock);
            }
        }
    }
}
